@@ -20,6 +20,7 @@
 #include <config.h>
 
 #include <stdlib.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <string.h>
@@ -34,6 +35,10 @@
 
 static gchar *tmp_dir = NULL;
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 /*
  * ev_dir_ensure_exists:
  * @dir: the directory name
@@ -139,7 +144,7 @@ ev_mkstemp (const char  *tmpl,
               return -1;
 
         name = g_build_filename (tmp, tmpl, NULL);
-        fd = g_mkstemp_full (name, O_CLOEXEC, 0600);
+        fd = g_mkstemp_full (name, O_RDWR | O_BINARY | O_CLOEXEC, 0600);
 
         if (fd == -1) {
 		int errsv = errno;
@@ -590,6 +595,19 @@ static const char *compressor_cmds[] = {
 #define N_ARGS      4
 #define BUFFER_SIZE 1024
 
+static void
+compression_child_setup_cb (gpointer fd_ptr)
+{
+        int fd = GPOINTER_TO_INT (fd_ptr);
+        int flags;
+
+        flags = fcntl (fd, F_GETFD);
+        if (flags >= 0 && (flags & FD_CLOEXEC)) {
+                flags &= ~FD_CLOEXEC;
+                fcntl (fd, F_SETFD, flags);
+        }
+}
+
 static gchar *
 compression_run (const gchar       *uri,
 		 EvCompressionType  type,
@@ -637,7 +655,8 @@ compression_run (const gchar       *uri,
 
 	if (g_spawn_async_with_pipes (NULL, argv, NULL,
 				      G_SPAWN_STDERR_TO_DEV_NULL,
-				      NULL, NULL, NULL,
+                                      compression_child_setup_cb, GINT_TO_POINTER (fd),
+                                      NULL,
 				      NULL, &pout, NULL, &err)) {
 		GIOChannel *in, *out;
 		gchar buf[BUFFER_SIZE];
