@@ -42,10 +42,6 @@
 
 #define THUMBNAIL_WIDTH 100
 
-/* The IconView doesn't scale nearly as well as the TreeView, so we arbitrarily
- * limit its use */
-#define MAX_ICON_VIEW_PAGE_COUNT 1500
-
 typedef struct _EvThumbsSize
 {
 	gint width;
@@ -62,7 +58,6 @@ typedef struct _EvThumbsSizeCache {
 struct _EvSidebarThumbnailsPrivate {
 	GtkWidget *swindow;
 	GtkWidget *icon_view;
-	GtkWidget *tree_view;
 	GtkAdjustment *vadjustment;
 	GtkListStore *list_store;
 	GHashTable *loading_icons;
@@ -75,7 +70,8 @@ struct _EvSidebarThumbnailsPrivate {
 
 	int rotation;
 	gboolean inverted_colors;
-
+	gboolean blank_first_dual_mode; /* flag for when we're using a blank first thumbnail
+					 * for dual mode with !odd_left preference. Issue #30 */
 	/* Visible pages */
 	gint start_page, end_page;
 };
@@ -104,6 +100,7 @@ static void         thumbnail_job_completed_callback       (EvJobThumbnail
 							    EvSidebarThumbnails     *sidebar_thumbnails);
 static void         ev_sidebar_thumbnails_reload           (EvSidebarThumbnails     *sidebar_thumbnails);
 static void         adjustment_changed_cb                  (EvSidebarThumbnails     *sidebar_thumbnails);
+static void         check_toggle_blank_first_dual_mode     (EvSidebarThumbnails     *sidebar_thumbnails);
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarThumbnails, 
                         ev_sidebar_thumbnails, 
@@ -229,37 +226,21 @@ ev_sidebar_thumbnails_page_is_in_visible_range (EvSidebarThumbnails *sidebar,
         GtkTreePath *path;
         GtkTreePath *start, *end;
         gboolean     retval;
+        GList *selection;
 
-        if (sidebar->priv->tree_view) {
-                GtkTreeSelection *selection;
-                GtkTreeIter       iter;
+        selection = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (sidebar->priv->icon_view));
+        if (!selection)
+                return FALSE;
 
-                selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar->priv->tree_view));
-                if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
-                        return FALSE;
+        path = (GtkTreePath *)selection->data;
 
-                path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->priv->list_store), &iter);
-                if (!gtk_tree_view_get_visible_range (GTK_TREE_VIEW (sidebar->priv->tree_view), &start, &end)) {
-                        gtk_tree_path_free (path);
-                        return FALSE;
-                }
-        } else {
-                GList *selection;
+        /* We don't handle or expect multiple selection. */
+        g_assert (selection->next == NULL);
+        g_list_free (selection);
 
-                selection = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (sidebar->priv->icon_view));
-                if (!selection)
-                        return FALSE;
-
-                path = (GtkTreePath *)selection->data;
-
-                /* We don't handle or expect multiple selection. */
-                g_assert (selection->next == NULL);
-                g_list_free (selection);
-
-                if (!gtk_icon_view_get_visible_range (GTK_ICON_VIEW (sidebar->priv->icon_view), &start, &end)) {
-                        gtk_tree_path_free (path);
-                        return FALSE;
-                }
+        if (!gtk_icon_view_get_visible_range (GTK_ICON_VIEW (sidebar->priv->icon_view), &start, &end)) {
+                gtk_tree_path_free (path);
+                return FALSE;
         }
 
         retval = gtk_tree_path_compare (path, start) >= 0 && gtk_tree_path_compare (path, end) <= 0;
@@ -299,10 +280,7 @@ ev_sidebar_thumbnails_get_property (GObject    *object,
 
 	switch (prop_id) {
 	case PROP_WIDGET:
-		if (sidebar->priv->tree_view)
-			g_value_set_object (value, sidebar->priv->tree_view);
-		else
-			g_value_set_object (value, sidebar->priv->icon_view);
+		g_value_set_object (value, sidebar->priv->icon_view);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -515,6 +493,9 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 
 	g_assert (start_page <= end_page);
 
+	if (priv->blank_first_dual_mode)
+		page--;
+
 	path = gtk_tree_path_new_from_indices (start_page, -1);
 	for (result = gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->list_store), &iter, path);
 	     result && page <= end_page;
@@ -615,23 +596,7 @@ adjustment_changed_cb (EvSidebarThumbnails *sidebar_thumbnails)
 
 	value = gtk_adjustment_get_value (priv->vadjustment);
 	
-	if (priv->tree_view) {
-		if (! gtk_widget_get_realized (priv->tree_view))
-			return;
-
-		gtk_tree_view_convert_tree_to_bin_window_coords (GTK_TREE_VIEW (priv->tree_view),
-								 0, (int) value,
-								 NULL, &wy1);
-		gtk_tree_view_convert_tree_to_bin_window_coords (GTK_TREE_VIEW (priv->tree_view),
-								 0, (int) (value + page_size),
-								 NULL, &wy2);
-		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
-					       1, wy1 + 1, &path,
-					       NULL, NULL, NULL);
-		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
-					       1, wy2 -1, &path2,
-					       NULL, NULL, NULL);
-	} else if (priv->icon_view) {
+	if (priv->icon_view) {
 		if (! gtk_widget_get_realized (priv->icon_view))
 			return;
 		if (! gtk_icon_view_get_visible_range (GTK_ICON_VIEW (priv->icon_view), &path, &path2))
@@ -729,41 +694,23 @@ ev_sidebar_icon_selection_changed (GtkIconView         *icon_view,
 	path = selected->data;
 	page = gtk_tree_path_get_indices (path)[0];
 
+	if (priv->blank_first_dual_mode) {
+		if (page == 0) {
+			gtk_icon_view_unselect_path (icon_view, path);
+			gtk_tree_path_free (path);
+			g_list_free (selected);
+			return;
+		}
+		page--;
+
+	}
+
 	gtk_tree_path_free (path);
 	g_list_free (selected);
 
 	ev_document_model_set_page (priv->model, page);
 }
 
-static void
-ev_sidebar_init_tree_view (EvSidebarThumbnails *ev_sidebar_thumbnails)
-{
-	EvSidebarThumbnailsPrivate *priv;
-	GtkTreeSelection *selection;
-	GtkCellRenderer *renderer;
-
-	priv = ev_sidebar_thumbnails->priv;
-	priv->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (priv->list_store));
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (ev_sidebar_tree_selection_changed), ev_sidebar_thumbnails);
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->tree_view), FALSE);
-	renderer = g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
-				 "xpad", 2,
-				 "ypad", 2,
-				 NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->tree_view), -1,
-						     NULL, renderer,
-						     "surface", 1,
-						     NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->tree_view), -1,
-						     NULL, gtk_cell_renderer_text_new (),
-						     "markup", 0, NULL);
-	gtk_container_add (GTK_CONTAINER (priv->swindow), priv->tree_view);
-	gtk_widget_show (priv->tree_view);
-}
-
 static void
 ev_sidebar_init_icon_view (EvSidebarThumbnails *ev_sidebar_thumbnails)
 {
@@ -796,18 +743,22 @@ ev_sidebar_init_icon_view (EvSidebarThumbnails *ev_sidebar_thumbnails)
 	g_signal_connect (priv->icon_view, "selection-changed",
 			  G_CALLBACK (ev_sidebar_icon_selection_changed), ev_sidebar_thumbnails);
 
+	g_signal_connect_swapped (priv->icon_view, "size-allocate",
+				  G_CALLBACK (check_toggle_blank_first_dual_mode), ev_sidebar_thumbnails);
+
+	g_signal_connect_data (priv->model, "notify::dual-page",
+			       G_CALLBACK (check_toggle_blank_first_dual_mode), ev_sidebar_thumbnails,
+			       NULL, G_CONNECT_SWAPPED | G_CONNECT_AFTER);
+
+	g_signal_connect_data (priv->model, "notify::dual-odd-left",
+			       G_CALLBACK (check_toggle_blank_first_dual_mode), ev_sidebar_thumbnails,
+			       NULL, G_CONNECT_SWAPPED | G_CONNECT_AFTER);
+
+	check_toggle_blank_first_dual_mode (ev_sidebar_thumbnails);
 	gtk_container_add (GTK_CONTAINER (priv->swindow), priv->icon_view);
 	gtk_widget_show (priv->icon_view);
 }
 
-static gboolean
-ev_sidebar_thumbnails_use_icon_view (EvSidebarThumbnails *sidebar_thumbnails)
-{
-	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
-
-	return (ev_document_get_n_pages (priv->document) <= MAX_ICON_VIEW_PAGE_COUNT);
-}
-
 static void
 ev_sidebar_thumbnails_device_scale_factor_changed_cb (EvSidebarThumbnails *sidebar_thumbnails,
                                                       GParamSpec          *pspec)
@@ -839,6 +790,7 @@ ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
 	guint signal_id;
 
 	priv = ev_sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (ev_sidebar_thumbnails);
+	priv->blank_first_dual_mode = FALSE;
 
 	priv->list_store = gtk_list_store_new (NUM_COLUMNS,
 					       G_TYPE_STRING,
@@ -876,16 +828,14 @@ static void
 ev_sidebar_thumbnails_set_current_page (EvSidebarThumbnails *sidebar,
 					gint                 page)
 {
-	GtkTreeView *tree_view;
 	GtkTreePath *path;
 
+	if (sidebar->priv->blank_first_dual_mode)
+		page++;
+
 	path = gtk_tree_path_new_from_indices (page, -1);
 
-	if (sidebar->priv->tree_view) {
-		tree_view = GTK_TREE_VIEW (sidebar->priv->tree_view);
-		gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
-		gtk_tree_view_scroll_to_cell (tree_view, path, NULL, FALSE, 0.0, 0.0);
-	} else if (sidebar->priv->icon_view) {
+	if (sidebar->priv->icon_view) {
 
 		g_signal_handlers_block_by_func
 			(sidebar->priv->icon_view,
@@ -1000,10 +950,7 @@ thumbnail_job_completed_callback (EvJobThumbnail      *job,
 			    -1);
         cairo_surface_destroy (surface);
 
-	if (ev_sidebar_thumbnails_use_icon_view (sidebar_thumbnails))
-		gtk_widget_queue_draw (priv->icon_view);
-	else
-		gtk_widget_queue_draw (priv->tree_view);
+	gtk_widget_queue_draw (priv->icon_view);
 }
 
 static void
@@ -1036,29 +983,11 @@ ev_sidebar_thumbnails_document_changed_cb (EvDocumentModel     *model,
 	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
 	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
 
-	/* Create the view widget, and remove the old one, if needed */
-	if (ev_sidebar_thumbnails_use_icon_view (sidebar_thumbnails)) {
-		if (priv->tree_view) {
-			gtk_container_remove (GTK_CONTAINER (priv->swindow), priv->tree_view);
-			priv->tree_view = NULL;
-		}
-
-		if (! priv->icon_view) {
-			ev_sidebar_init_icon_view (sidebar_thumbnails);
-			g_object_notify (G_OBJECT (sidebar_thumbnails), "main_widget");
-		} else {
-			gtk_widget_queue_resize (priv->icon_view);
-		}
+	if (! priv->icon_view) {
+		ev_sidebar_init_icon_view (sidebar_thumbnails);
+		g_object_notify (G_OBJECT (sidebar_thumbnails), "main_widget");
 	} else {
-		if (priv->icon_view) {
-			gtk_container_remove (GTK_CONTAINER (priv->swindow), priv->icon_view);
-			priv->icon_view = NULL;
-		}
-
-		if (! priv->tree_view) {
-			ev_sidebar_init_tree_view (sidebar_thumbnails);
-			g_object_notify (G_OBJECT (sidebar_thumbnails), "main_widget");
-		}
+		gtk_widget_queue_resize (priv->icon_view);
 	}
 
 	/* Connect to the signal and trigger a fake callback */
@@ -1145,3 +1074,143 @@ ev_sidebar_thumbnails_page_iface_init (EvSidebarPageInterface *iface)
 	iface->set_model = ev_sidebar_thumbnails_set_model;
 	iface->get_label = ev_sidebar_thumbnails_get_label;
 }
+
+static gboolean
+iter_is_blank_thumbnail (GtkTreeModel *tree_model,
+			 GtkTreeIter  *iter)
+{
+	cairo_surface_t *surface = NULL;
+	EvJob *job = NULL;
+	gboolean thumbnail_set = FALSE;
+
+	gtk_tree_model_get (tree_model, iter,
+			    COLUMN_SURFACE, &surface,
+			    COLUMN_THUMBNAIL_SET, &thumbnail_set,
+			    COLUMN_JOB, &job, -1);
+
+	/* The blank thumbnail item can be distinguished among all
+	 * other items in the GtkIconView as it's the only one which
+	 * has the COLUMN_SURFACE as NULL while COLUMN_THUMBNAIL_SET
+	 * is set to TRUE. */
+	return surface == NULL && job == NULL && thumbnail_set;
+}
+
+/* Returns the total horizontal(left+right) width of thumbnail frames.
+ * As it was added in ev_document_misc_render_thumbnail_frame() */
+static gint
+ev_sidebar_thumbnails_frame_horizontal_width (EvSidebarThumbnails *sidebar)
+{
+        GtkWidget *widget;
+        GtkStyleContext *context;
+        GtkStateFlags state;
+        GtkBorder border = {0, };
+        gint offset;
+
+        widget = GTK_WIDGET (sidebar);
+        context = gtk_widget_get_style_context (widget);
+        state = gtk_widget_get_state_flags (widget);
+
+        gtk_style_context_save (context);
+
+        gtk_style_context_add_class (context, "page-thumbnail");
+        gtk_style_context_get_border (context, state, &border);
+        offset = border.left + border.right;
+
+        gtk_style_context_restore (context);
+
+        return offset;
+}
+
+/* Returns whether the thumbnail sidebar is currently showing
+ * items in a two columns layout */
+static gboolean
+ev_sidebar_thumbnails_is_two_columns (EvSidebarThumbnails *sidebar)
+{
+        EvSidebarThumbnailsPrivate *priv;
+        GtkWidget *window;
+        GtkIconView *icon_view;
+        gint sidebar_width, two_columns_width, three_columns_width;
+        gint margin, column_spacing, item_padding, thumbnail_width;
+        static gint frame_horizontal_width;
+
+        priv = sidebar->priv;
+        icon_view = GTK_ICON_VIEW (priv->icon_view);
+
+        ev_thumbnails_size_cache_get_size (priv->size_cache, 0,
+                                           priv->rotation,
+                                           &thumbnail_width, NULL);
+
+        margin = gtk_icon_view_get_margin (icon_view);
+        column_spacing = gtk_icon_view_get_column_spacing (icon_view);
+        item_padding = gtk_icon_view_get_item_padding (icon_view);
+        frame_horizontal_width = ev_sidebar_thumbnails_frame_horizontal_width (sidebar);
+
+        two_columns_width = 2 * margin +
+                            4 * item_padding +
+                            2 * frame_horizontal_width +
+                            2 * thumbnail_width +
+                            column_spacing;
+
+        three_columns_width = 2 * margin +
+                              6 * item_padding +
+                              3 * frame_horizontal_width +
+                              3 * thumbnail_width +
+                              2 * column_spacing;
+
+        if (priv->width == 0) {
+                window = gtk_widget_get_toplevel (GTK_WIDGET (sidebar));
+                sidebar_width = ev_window_get_metadata_sidebar_size (EV_WINDOW (window));
+        } else {
+                sidebar_width = priv->width;
+        }
+
+        return sidebar_width >= two_columns_width &&
+               sidebar_width < three_columns_width;
+}
+
+/* Checks whether the conditions for 'blank first dual mode' are met,
+ * and activates/deactivates the mode accordingly. */
+static void
+check_toggle_blank_first_dual_mode (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv;
+	GtkTreeModel *tree_model;
+	GtkTreeIter first;
+        gboolean should_be_enabled;
+
+        priv = sidebar_thumbnails->priv;
+
+	should_be_enabled = ev_document_model_get_dual_page (priv->model) &&
+	    !ev_document_model_get_dual_page_odd_pages_left (priv->model) &&
+	     ev_sidebar_thumbnails_is_two_columns (sidebar_thumbnails);
+
+	if (should_be_enabled && !priv->blank_first_dual_mode) {
+		/* Do enable it */
+		tree_model = GTK_TREE_MODEL (priv->list_store);
+
+		if (!gtk_tree_model_get_iter_first (tree_model, &first))
+			return;
+
+		priv->blank_first_dual_mode = TRUE;
+		if (iter_is_blank_thumbnail (tree_model, &first))
+			return; /* extra check */
+
+		gtk_list_store_insert_with_values (priv->list_store, &first, 0,
+						   COLUMN_SURFACE, NULL,
+						   COLUMN_THUMBNAIL_SET, TRUE,
+						   COLUMN_JOB, NULL,
+						   -1);
+	} else if (!should_be_enabled && priv->blank_first_dual_mode) {
+		/* Do disable it */
+		tree_model = GTK_TREE_MODEL (priv->list_store);
+
+		if (!gtk_tree_model_get_iter_first (tree_model, &first))
+			return;
+
+		priv->blank_first_dual_mode = FALSE;
+		if (!iter_is_blank_thumbnail (tree_model, &first))
+			return; /* extra check */
+
+		gtk_list_store_remove (priv->list_store, &first);
+	}
+}
